<script setup>
import AppLayoutWater from '@/Layouts/AppLayoutWater.vue';
import Welcome from '@/Jetstream/Welcome.vue';
import Datepicker from '@vuepic/vue-datepicker';
import JetDropdown from '@/Jetstream/Dropdown.vue';
import JetDropdownLink from '@/Jetstream/DropdownLink.vue';
import '@vuepic/vue-datepicker/dist/main.css';
import { startOfDay, endOfDay, subDays, endOfMonth, endOfYear, startOfMonth, startOfYear, subMonths } from 'date-fns';
import Highcharts from 'highcharts';
import VueMultiselect from 'vue-multiselect';
import mapboxgl from 'mapbox-gl';
import 'mapbox-gl/dist/mapbox-gl.css';
import JetModal from '@/Jetstream/Modal.vue';
import { Link } from '@inertiajs/inertia-vue3';
import DK_Button from '@/components/dk_Button.vue';
import DK_Input from '@/components/dk_Input.vue';

</script>
<style>
table.dataTable thead th,
table.dataTable thead td {
    padding: 5px 10px;
    font-size: 0.9rem !important;
    border-bottom: 1px solid rgba(200, 200, 200, 0.75) !important;
}

.dataTables_wrapper .dataTables_paginate .paginate_button.disabled,
.dataTables_wrapper .dataTables_paginate .paginate_button.disabled:hover,
.dataTables_wrapper .dataTables_paginate .paginate_button.disabled:active {
    cursor: default;
    color: rgba(200, 200, 200, 0.9) !important;
    border: 1px solid transparent;
    background: transparent;
    box-shadow: none;
}

.dataTables_wrapper .dataTables_paginate .paginate_button {
    box-sizing: border-box;
    display: inline-block;
    min-width: 1.5em;
    padding: 0.5em 1em;
    margin-left: 2px;
    text-align: center;
    text-decoration: none !important;
    cursor: pointer;
    cursor: hand;
    color: rgba(200, 200, 200, 0.9) !important;
    border: 0;
    border-radius: 0.375rem;
}

.dataTables_wrapper .dataTables_paginate .paginate_button:hover {
    color: rgba(200, 200, 200, 1) !important;
    background-color: rgba(0, 152, 219, 0.1) !important;
    background-image: unset !important;
    border: 0;
}

.dp__action_row {
    border-radius: 0.5rem;
}

.dp__input_reg {
    background: none;
    border-width: 1px!important;
    border-color: rgba(200, 200, 200, 0.9)!important;
    border-radius: 0.5rem;

}

.dp__theme_light {
    --dp-background-color: rgba(0, 57, 93, 0.95);
    --dp-text-color: #e3e3e3;
    --dp-hover-color: rgba(0, 255, 255, 0.2);
    --dp-hover-text-color: #FFFAFA;
    --dp-hover-icon-color: #e3e3e3;
    --dp-primary-color: rgba(0, 255, 255, 0.5);
    --dp-primary-text-color: #FFFAFA;
    --dp-secondary-color: #959595;
    --dp-border-color: #FFFAFA;
    --dp-menu-border-color: #FFFAFA;
    --dp-border-color-hover: #FFFAFA;
    --dp-disabled-color: #959595;
    --dp-scroll-bar-background: #f3f3f3;
    --dp-scroll-bar-color: #959595;
    --dp-success-color: rgba(5, 255, 243, 0.7);
    --dp-success-color-disabled: rgba(5, 255, 243, 0.7);
    --dp-icon-color: #e3e3e3;
    --dp-danger-color: #e6002e;
    --dp-highlight-color: rgba(25, 118, 210, 0.1);
}

.dp__pointer {
    padding-top: 0.25rem;
    padding-bottom: 0.25rem;
}
</style>

<template>
    <AppLayoutWater title="Readings">
        <div v-if="this.screen_width >= 768" style="height: calc(100vh - 4.75rem);" class="bg-darkblue-background -z-1 overflow-y-scroll select-none">
            <!-- TIME INTERVAL SELECTOR -->
            <div class="mx-6 text-xs text-center text-gray-light flex justify-between">
                <div class="flex flex-col justify-start items-start w-full mb-6 pt-4">
                    <div class="justify-between self-center items-baseline flex w-full"> <!--id="routeprofile"-->
                        <p class="text-white-snow font-light text-left text-2xl w-full">{{ this.gateway[0].name }}&nbsp;(Id:&nbsp;{{ this.gateway[0].id }}) </p>

                        <div class="flex gap-2 items-end">
                            <div class="flex items-start justify-end">
                                <Link onclick="window.emitter.emit('changeLoaderStatus', true, 'orbit')" :href="route('meterProfile', { id:  this.gateway[0].id })" >
                                    <DK_Button color="blue" :fixedSize="true">
                                        {{ this.$t("general.Profile") }}
                                    </DK_Button>
                                </Link>
                            </div>
                            <div class="flex items-start justify-end">
                                <DK_Button class="pt-6" color="blue" :fixedSize="true" onclick="window.emitter.emit('changeLoaderStatus', true, 'orbit'); window.history.go(-1)">
                                    {{ this.$t("general.Back") }}
                                </DK_Button>
                            </div>
                        </div>
                    </div>

                    <div class="mt-4 inline-flex gap-6">

                        <Datepicker id="interval_date" class="w-[22rem] justify-self-end self-end" v-model="date" vertical
                            :presetRanges="this.presetRanges" range @update:modelValue="methodUpdateDate" :clearable="false"
                            :format="this.format"></Datepicker>

                    </div>
                    <div v-bind:class="{ 'hidden': this.data.dates }" class="flex gap-2 whitespace-nowrap items-baseline mt-4">
                        <i class="fa-solid fa-triangle-exclamation" style="font-size:0.8rem; color:rgba(224,191,52,1);"></i>
                        <p class="font-light text-sm text-white-snow flex whitespace-nowrap">
                            {{ this.$t('text.No records were found for the device') }}
                        </p>
                    </div>
                </div>

            </div>

            <div class="grid 2xl:grid-cols-8 grid-cols-6 gap-x-6 px-6 mb-6" v-for="(chart, index) in this.Charts" :key="index"
                v-bind:class="{ 'hidden': chart == undefined }">


                <div class="2xl:col-span-7 col-span-5 baltoro-container">

                    <!-- CHART -->
                    <div class="px-10 pt-4">
                        <p class="text-gray-light font-light text-2xl mb-4">{{ (chart != undefined) ?
                            this.getName(chart.unit) :
                            ''
                        }}</p>
                        <div class="2xl:col-span-7 col-span-5 cursor-crosshair pb-4"
                            v-bind:class="{' h-[11rem]': this.charts_size.includes(index), 'h-[22rem]': !this.charts_size.includes(index) }">
                            <div class="h-full w-full" :id="'id_chart' + index"></div>
                        </div>
                    </div>

                </div>

                <!-- SIDEMENU -->
                <div class="col-span-1 grid content-start text-center text-gray-light text-xs gap-6"
                    v-if="this.data.type_name == 'MODBUS'">

                    <VueMultiselect v-if="this.check_VMoptions(index)" :id="'id_VueMultiselect_data_type' + index"
                        v-model="this.VMmodel[index]" :options="this.VMoptions[index]" :close-on-select="true"
                        :multiple="false" placeholder='Select sensor' @select="selectMultiselectSensor" label="name"
                        track-by="name" :allow-empty="false" />

                    <div class="m-0 p-0 baltoro-container py-1">
                        <p class="text-cyan-text font-normal py-1">{{ this.$t("water." + "maxAmpLang") }}</p>
                        <div class="w-full inline-flex justify-center items-end">
                            <p class="text-3xl font-sevensegment">
                                {{ this.max_amplitude[index] }}
                            </p>
                            &nbsp;
                            <p class="text-xl pb-0.5">{{ (chart != undefined) ? chart.unit : '' }}</p>
                        </div>
                    </div>

                    <div class="m-0 gap-2 grid text-center baltoro-container py-1">

                        <p class="col-span-2 text-cyan-text font-normal py-1">{{ this.$t("general." + "Summary statistics")
                        }}
                        </p>

                        <div class="col-span-2 contents">
                            <p class="self-center">{{ this.$t("general." + "Max") }}</p>
                            <div class="inline-flex justify-center items-end">
                                <p class="text-2xl font-sevensegment self-center">
                                    {{ this.max[index] }}
                                </p>
                                &nbsp;
                                <p class="text-base pb-0.5">{{ (chart != undefined) ? chart.unit : '' }}</p>
                            </div>
                        </div>

                        <div class="col-span-2 contents">
                            <p class="self-center">{{ this.$t("general." + "Avg") }}</p>
                            <div class="inline-flex justify-center items-end">
                                <p class="text-2xl font-sevensegment self-center">
                                    {{ this.avg[index] }}
                                </p>
                                &nbsp;
                                <p class="text-base pb-0.5">{{ (chart != undefined) ? chart.unit : '' }}</p>
                            </div>
                        </div>

                        <div class="col-span-2 contents">
                            <p class="self-center">{{ this.$t("general." + "Min") }}</p>
                            <div class="inline-flex justify-center items-end">
                                <p class="text-2xl font-sevensegment self-center">
                                    {{ this.min[index] }}
                                </p>
                                &nbsp;
                                <p class="text-base pb-0.5">{{ (chart != undefined) ? chart.unit : '' }}</p>
                            </div>
                        </div>

                    </div>
                </div>
            </div>

            <div class="grid 2xl:grid-cols-8 grid-cols-6 gap-x-6 px-6">

                <div class="2xl:col-span-8 col-span-6 pt-5 mb-6 px-10 baltoro-container">

                    <table id="basicTable" class="row-border">
                        <thead class="bg-transparent text-gray-light">
                            <tr>
                                <th>{{ this.$t("general." + "Date") }}</th>
                                <th v-for="(measures, index) in this.data.measures" :key="index">{{ this.data.names[index] +
                                    ' '
                                    + this.data.units[index] }}</th>
                            </tr>
                        </thead>
                        <tbody class="text-center">
                        </tbody>
                    </table>

                </div>

            </div>

        </div>

        <div v-else style="height: calc(100vh - 4.75rem);" class="bg-darkblue-background -z-1 cursor-default select-none overflow-x-hidden">

            <div class="w-full flex flex-col justify-center items-start pt-2 pb-1 gap-2">
                <div class="w-full flex flex-row justify-between px-3">
                    <div class="w-full flex flex-col justify-start">
                        <div class="flex flex-row justify-start">
                            <p class="text-white-snow font-light text-2xl text-left">{{ this.gateway[0].name }}</p>
                        </div>
                        <p class="text-white-snow font-light text-sm text-left">{{ this.gateway[0].id }}</p>
                    </div>

                    <div class="flex justify-end gap-2">
                        <div class="flex items-center justify-center">
                            <Link onclick="window.emitter.emit('changeLoaderStatus', true, 'orbit')" :href="route('meterProfile', { id:  this.gateway[0].id })" >
                                <button type="button" class="text-white-snow button-blue font-medium rounded-lg text-sm p-1.5 text-center inline-flex items-center">
                                    <i class="fa-solid fa-gears" style="font-size:0.75rem; color:#FFFAFA;"></i>
                                </button>
                            </Link>
                        </div>
                        <div class="flex items-center justify-center">
                            <Link onclick="window.emitter.emit('changeLoaderStatus', true, 'orbit'); window.history.go(-1)">
                                <button type="button" class="text-white-snow button-blue font-medium rounded-lg text-sm p-1.5 text-center inline-flex items-center">
                                    <i class="fa-solid fa-arrow-left" style="font-size:0.75rem; color:#FFFAFA;"></i>
                                </button>
                            </Link>
                        </div>
                    </div>
                </div>

                <div class="w-full px-3">

                    <Datepicker id="interval_date" class="w-full justify-self-end self-end" v-model="date" vertical
                        :presetRanges="this.presetRanges" range @update:modelValue="methodUpdateDate" :clearable="false"
                        :format="this.methodFormatDatetoDatepicker"></Datepicker>

                </div>

                <div v-bind:class="{ 'hidden':this.data.dates}" class="flex gap-1.5 whitespace-nowrap items-baseline w-full justify-center px-3">
                    <i class="fa-solid fa-triangle-exclamation" style="font-size:0.8rem; color:rgba(224,191,52,1);"></i>
                    <p class="font-light text-sm text-white-snow flex whitespace-nowrap">
                        {{ this.$t('text.No records were found for the device') }}
                    </p>
                </div>

                <div class="w-full flex overflow-x-auto snap-mandatory snap-x pb-2 gap-3 px-3" :style="{ 'scroll-snap-align': 'start'}">

                    <div class="snap-center shrink-0 flex flex-col snap-always gap-2 w-full"
                    v-for="(chart, index) in this.Charts" :key="index"  v-bind:class="{ 'hidden': chart == undefined }">

                        <div class="w-full p-2 baltoro-container ">

                            <!-- CHART -->
                            <p class="text-white-snow font-light text-lg mb-2 whitespace-nowrap w-full text-center"> {{ (chart != undefined) ?
                                this.getName(chart.unit) :
                                ''
                            }}</p>

                            <div class="2xl:col-span-7 col-span-5 cursor-crosshair"
                                v-bind:class="{'h-[11rem]': this.charts_size.includes(index), 'h-[22rem]': !this.charts_size.includes(index) }">
                                <div class="h-full w-full" :id="'id_chart' + index"></div>
                            </div>
                        </div>

                        <div class="baltoro-container w-full flex flex-row justify-between items-center p-2">
                            <p class="text-cyan-text font-light text-md whitespace-nowrap w-full text-center">{{ this.$t("water." + "maxAmpLang") }}</p>
                            <div class="w-full inline-flex justify-center items-end text-gray-light">
                                <p class="text-2xl font-sevensegment">
                                    {{ this.max_amplitude[index] }}
                                </p>
                                <p class="text-xl pb-0.5">{{ (chart != undefined) ? chart.unit : '' }}</p>
                            </div>
                        </div>



                        <div class="grid text-center baltoro-container text-gray-light p-2" v-if="this.data.type_name == 'MODBUS'">

                            <VueMultiselect v-if="this.check_VMoptions(index)" :id="'id_VueMultiselect_data_type' + index"
                                v-model="this.VMmodel[index]" :options="this.VMoptions[index]" :close-on-select="true"
                                :multiple="false" placeholder='Select sensor' @select="selectMultiselectSensor" label="name"
                                track-by="name" :allow-empty="false" />

                            <p class="col-span-2 text-cyan-text font-light text-md whitespace-nowrap w-full text-center py-1">{{ this.$t("general." + "Summary statistics")}}</p>

                            <div class="col-span-2 contents">
                                <p class="self-center">{{ this.$t("general." + "Max") }}</p>
                                <div class="inline-flex justify-center items-end">
                                    <p class="text-2xl font-sevensegment self-center">
                                        {{ this.max[index] }}
                                    </p>
                                    <p class="pb-1"> {{ (chart != undefined) ? chart.unit : '' }}</p>
                                </div>
                            </div>

                            <div class="col-span-2 contents">
                                <p class="self-center">{{ this.$t("general." + "Avg") }}</p>
                                <div class="inline-flex justify-center items-end">
                                    <p class="text-2xl font-sevensegment self-center">
                                        {{ this.avg[index] }}
                                    </p>
                                    <p class="pb-1"> {{ (chart != undefined) ? chart.unit : '' }}</p>
                                </div>
                            </div>

                            <div class="col-span-2 contents">
                                <p class="self-center">{{ this.$t("general." + "Min") }}</p>
                                <div class="inline-flex justify-center items-end">
                                    <p class="text-2xl font-sevensegment self-center">
                                        {{ this.min[index] }}
                                    </p>
                                    <p class="pb-1"> {{ (chart != undefined) ? chart.unit : '' }}</p>
                                </div>
                            </div>

                        </div>

                        <div class="2xl:col-span-8 col-span-6 pt-5 mb-6 px-3 baltoro-container w-full">

                            <table :id="'basicTable_' + index" class="row-border">
                                <thead class="bg-transparent text-gray-light">
                                    <tr>
                                        <th>{{ this.$t("general." + "Date") }}</th>
                                        <th >{{ this.data.names[Object.keys(this.data.names)[index]]+
                                            ' '
                                            + this.data.units[Object.keys(this.data.units)[index]] }}</th>
                                    </tr>
                                </thead>
                                <tbody class="text-center">
                                </tbody>
                            </table>

                        </div>


                    </div>
                </div>
            </div>


        </div>
    </AppLayoutWater>
</template>

<script>
export default {
    props: ['data', 'gateway', 'user'],

    data() {
        return {
            Back: localStorage.getItem("Back"),
            presetRanges: [
                { label: this.$t("general." + "Today"), range: [startOfDay(new Date()), new Date()] },
                { label: this.$t("general." + "Yesterday"), range: [startOfDay(subDays(new Date(), 1)), endOfDay(subDays(new Date(), 1))] },
                { label: this.$t("general." + "Last 7 Days"), range: [new Date().setDate(new Date().getDate() - 7), new Date().setDate(new Date().getDate())] },
                { label: this.$t("general." + "Last 14 Days"), range: [new Date().setDate(new Date().getDate() - 14), new Date().setDate(new Date().getDate())] },
                { label: this.$t("general." + "Last 30 Days"), range: [new Date().setDate(new Date().getDate() - 30), new Date().setDate(new Date().getDate())] },
                { label: this.$t("general." + "This month"), range: [startOfMonth(new Date()), new Date()] },
                { label: this.$t("general." + "Last month"), range: [startOfMonth(subMonths(new Date(), 1)), endOfMonth(subMonths(new Date(), 1))] }
            ],

            formatDate: (date, utc) => {

                date = new Date(date);
                // utc en 0 para formatear hora local del usuario, 1 para formatear UTC
                if (utc == 0) { date.setHours(date.getHours() + this.user.timezone_offset) }
                else if (utc == -1) { date.setHours(date.getHours() - this.user.timezone_offset) }

                return (
                    date.getFullYear() + "-" +
                    ((date.getMonth() + 1 + '').length < 2 ? "0" + (date.getMonth() + 1) : (date.getMonth() + 1)) + "-" +
                    ((date.getDate() + '').length < 2 ? "0" + date.getDate() : date.getDate()) + " " +
                    ((date.getHours() + '').length < 2 ? "0" + date.getHours() : date.getHours()) + ":" +
                    ((date.getMinutes() + '').length < 2 ? "0" + date.getMinutes() : date.getMinutes()) + ":" +
                    ((date.getSeconds() + '').length < 2 ? "0" + date.getSeconds() : date.getSeconds())
                );
            },

            formatTime: (date_state_change, diff) => {

                let duration_ms;
                if (diff == true) {
                    duration_ms = new Date() - new Date(date_state_change);
                } else {
                    duration_ms = date_state_change * 1000;
                }

                const diff_D = Math.floor(duration_ms / (1000 * 60 * 60 * 24));
                const diff_H = Math.floor(
                    (duration_ms % (1000 * 60 * 60 * 24)) / (1000 * 60 * 60)
                );
                const diff_M = Math.floor(
                    (duration_ms % (1000 * 60 * 60)) / (1000 * 60)
                );

                let duration = "";

                if (this.user.language == "es" && diff) {
                    duration = this.$t("general.for") + " ";
                }

                if (diff_D > 0) {
                    duration += this.$t("general.morethan") + " " + diff_D + " ";
                    if (diff_D > 1) {
                        duration += this.$t("general.days");
                    } else {
                        duration += this.$t("general.day");
                    }
                } else {
                    if (diff_H > 0) {
                        duration += diff_H;
                        if (diff_M > 0) {
                            duration += this.$t("general.h");
                        } else if (diff_H > 1) {
                            duration += " " + this.$t("general.hours");
                        } else {
                            duration += " " + this.$t("general.hour");
                        }

                        if (diff_M != 0) {
                            duration += " " + this.$t("general.and") + " ";
                        }
                    }

                    if (diff_M > 0) {
                        duration += diff_M;
                        if (diff_H != 0 && diff_M > 1) {
                            duration += this.$t("general.mins");
                        } else if (diff_H != 0 && diff_M == 1) {
                            duration += this.$t("general.min");
                        } else if (diff_M > 1) {
                            duration += " " + this.$t("general.minutes");
                        } else {
                            duration += " " + this.$t("general.minute");
                        }
                    }
                }

                if (diff_H == 0 && diff_H == 0 && diff_M == 0) {
                    duration = "0 " + this.$t("general.minutes");
                }

                if (this.user.language == "en" && diff) {
                    duration += " " + this.$t("general.ago");
                }

                return duration;
            },

            colors: ["#05fff3", "#FF29D9", "#FFC100", "#6DFF33"],
            date: undefined,
            default_Chart: undefined,
            Charts: [undefined, undefined, undefined, undefined, undefined, undefined, undefined,],
            Highcharts: require('highcharts'),
            VMoptions: new Array(),
            VMmodel: new Array(),
            max: new Array(),
            min: new Array(),
            avg: new Array(),
            max_amplitude: new Array(),
            charts_size: new Array(),
            gateway_update: new Object(),
            Mapconfig: false,
            screen_width: 0,

        }
    },

    beforeCreate: function () {
        this.$i18n.locale = this.user.language
    },

    beforeMount() {
        this.screen_width = window.screen.width;

        this.Charts.forEach((chart, index) => {
            this.VMoptions[index] = new Array();
        })

        this.VMoptions[0] = new Array();

        let formatDate = this.formatDate;

        if(this.data.dates){

            this.date = [
                new Date(formatDate(this.data.dates[this.data.dates.length - 1], 0)),
                new Date(formatDate(this.data.dates[0], 0))
            ];
        }else{
            let enddate = new Date();
            enddate.setDate(enddate.getDate() - 1);
            let startdate = new Date();

            this.date = [startdate, enddate];
        }

        this.gateway_update = {
            name: this.gateway[0].name,
            GPS_lat: parseFloat(this.gateway[0].GPS_lat).toFixed(6),
            GPS_lng: parseFloat(this.gateway[0].GPS_lng).toFixed(6)
        }

    },

    mounted() {


        window.emitter.emit('changeLoaderStatus', false, 'orbit')

        let export_title = this.gateway[0].name + '_' + this.gateway[0].id + '_' + this.formatDate(new Date(), 1)

        if(this.screen_width >=768){

            this.table = $('#basicTable').DataTable({
                language: {
                    url: '/assets/i18n/' + this.user.language + '.json'
                },
                order: [[0, "desc"]],
                dom: '<"row"<"col-sm-6"B><"col-sm-6">>t<"row"<"col-sm-6"i><"col-sm-6"p>><"clear">',
                ordering: true,
                pageLength: 10,
                buttons: [
                    {
                        extend: 'csv',
                        text: 'CSV',
                        title: export_title,
                        className: 'btn btn-brand',
                        exportOptions: {
                            columns: ':visible'
                        }
                    },
                    {
                        extend: 'excel',
                        text: 'Excel',
                        title: export_title,
                        className: 'btn btn-brand',
                        exportOptions: {
                            columns: ':visible'
                        }
                    },
                    {
                        extend: 'print',
                        text: 'Print',
                        title: export_title,
                        className: 'btn btn-brand',
                        exportOptions: {
                            columns: ':visible'
                        }
                    },
                ]
            });

        }else{

            this.table = new Array( this.data.measures.length);


            Object.values(this.data.measures).forEach((measures,  index) => {

                var tableId = 'basicTable_' + index;

                this.table[index] = $('#' + tableId).DataTable({
                    language: {
                        url: '/assets/i18n/' + this.user.language + '.json'
                    },
                    order: [[0, "desc"]],
                    dom: '<"row"<"col-sm-6"B><"col-sm-6">>t<"row"<"col-sm-6"i><"col-sm-6"p>><"clear">',
                    ordering: true,
                    pageLength: 10,
                    buttons: [
                        {
                            extend: 'csv',
                            text: 'CSV',
                            title: export_title,
                            className: 'btn btn-brand',
                            exportOptions: {
                                columns: ':visible'
                            }
                        },
                        {
                            extend: 'excel',
                            text: 'Excel',
                            title: export_title,
                            className: 'btn btn-brand',
                            exportOptions: {
                                columns: ':visible'
                            }
                        },
                        {
                            extend: 'print',
                            text: 'Print',
                            title: export_title,
                            className: 'btn btn-brand',
                            exportOptions: {
                                columns: ':visible'
                            }
                        },
                    ]
                });

            })

        }
        this.rechargePage();

    },

    methods: {


        rechargePage() {

            document.addEventListener("DOMContentLoaded", function() {
                const scrollContainer = document.querySelector(".w-full.flex.overflow-x-auto");
                if (scrollContainer) {
                    scrollContainer.scrollTo(0, 0); // Ajustar el scroll a la posición inicial
                }
            });

            let formatDate = this.formatDate;

            const Highcharts = require('highcharts');

            this.charts_size = new Array();

            let Charts = [undefined, undefined, undefined];

            let default_Chart = {
                chart: {
                    zoomType: 'x',
                    backgroundColor: 'transparent',
                    spacingTop: 20,
                    animation: {
                        duration: 150
                    },
                resetZoomButton: {
                    position: {
                        align: 'left',
                        verticalAlign: 'top',
                        x: 0,
                        y: 2.5
                    },
                    theme: {
                        r: 5.6,
                    }
                }
                },
                accessibility: {
                    enabled: false
                },
                title: {
                    text: ''
                },
                xAxis: {
                    lineColor: '#647F8C',
                    visible: true,
                    labels: {
                        style: {
                            color: 'rgba(200, 200, 200, 0.9)',
                            opacity: 0.7,
                        }
                    },
                    type: "datetime",
                    dateTimeLabelFormats: {
                        //millisecond: '%H:%M:%S.%L',
                        //second: '%H:%M:%S',
                        //minute: '%H:%M',
                        hour: '%H:%M',
                        day: '%m-%d',
                        week: '%m-%d',
                        month: '%Y-%m',
                        year: '%Y'
                    },
                    time: {
                        timezoneOffset: new Date().getTimezoneOffset(), // Configura la zona horaria actual
                        useUTC: false // Utiliza hora local
                    }
                },
                yAxis: new Array(),
                exporting: {
                    enabled: false
                },
                credits: {
                    enabled: false
                },
                legend: {
                    enabled: true,
                    itemStyle: {
                        color: 'rgba(200, 200, 200, 0.9)',
                        opacity: 0.75,
                        fontWeight: 'light'
                    },
                    itemHoverStyle: {
                        color: 'rgba(200, 200, 200, 0.9)',
                        opacity: 1,
                        fontWeight: 'light'
                    }
                },
                tooltip: {
                    shared: true,
                    split: false,
                    backgroundColor: 'rgba(7,82,103,0.95)',
                    borderRadius: 12,
                    borderWidth: 1,
                    borderColor: 'rgba(250, 250, 250, 0.65)',
                    padding: 6,
                    formatter: function () {
                        var s = '<b>' + '<p style="font-size:12px; line-height: 12px; font-weight: 300; color:rgba(250, 250, 250, 0.95)">' + formatDate(this.x, -1) + '</p><br/>';
                        $.each(this.points, function () {
                            s += '<p style="font-size:12px; line-height: 12px; font-weight: 400; color:' + this.series.color + '">' + this.series.name + ' </p><p style="font-size:12px; line-height: 12px; font-weight: 400; color:' + this.series.color + '">' + this.y +
                                ' ' + this.series.userOptions.tooltip.valueSuffix + '</p><br/>';
                        });

                        return s;
                    },
                },
            };

            if (this.data.type_name == "MODBUS") {

                let sensor_num;

                if (this.data.measures) {
                    sensor_num = Object.keys(this.data.measures).length
                } else {
                    sensor_num = 1;
                }

                let units = new Array(sensor_num);
                let names = new Array(sensor_num);
                let data = new Array(sensor_num);
                let dates;

                for (let sensor in this.data.measures) {

                    dates = new Array();
                    data[sensor] = new Array();

                    names[sensor] = this.data.names[sensor];
                    units[sensor] = this.data.units[sensor];

                    let newChart = true;

                    Charts.forEach((chart, index) => {
                        if (chart != undefined) {
                            if (chart.unit == units[sensor]) {
                                newChart = false;
                                Charts[index].sensors.push(sensor)
                            }
                        }
                    })

                    if (newChart == true) {

                        let index = Charts.indexOf(undefined);

                        Charts[index] = {
                            unit: units[sensor],
                            highcharts: default_Chart,
                            sensors: [sensor],
                            id: 'id_chart' + index,
                        }
                    }

                    this.data.dates.forEach((date, index) => {
                        dates.push(new Date((date + ' GMT')).getTime() ); //+ this.user.timezone_offset * 3600000
                    })

                    this.data.measures[sensor].forEach((value, index) => {
                        data[sensor].push(parseFloat(value));
                    })

                }

                for (let i_chart = 0; i_chart < Charts.length; i_chart++) {

                    if (Charts[i_chart] != undefined) {

                        Charts[i_chart].highcharts.yAxis = (
                            {
                                id: 0,
                                gridLineColor: '#647F8C',
                                title: {
                                    text: ''
                                },
                                labels: {
                                    formatter: function () {
                                        return this.pos + ' ' + Charts[i_chart].unit
                                    },
                                    style: {
                                        color: 'rgba(200, 200, 200, 0.9)',
                                        opacity: 0.7,
                                    }
                                },
                                visible: true,
                            }
                        );

                        Charts[i_chart].highcharts.series = new Array();

                        data.forEach((sensor, index) => {

                            if (Charts[i_chart].sensors.includes(index.toString())) {

                                let highcharts_data = new Array(sensor.length);

                                sensor.forEach((measure, i) => {
                                    highcharts_data[i] = [dates[i], measure];
                                })

                                Charts[i_chart].highcharts.tooltip.valueSuffix = units[index];

                                Charts[i_chart].highcharts.series.push({
                                    name: names[index],
                                    type: 'spline',
                                    yAxis: 0,
                                    data: highcharts_data.filter(point => point !== undefined && point !== null).reverse(),
                                    color: this.colors[(Charts[i_chart].sensors.indexOf(index.toString()) % 4)],
                                    lineWidth: 1.5,
                                    zIndex: 1,
                                    marker: {
                                        enabled: false
                                    },
                                    tooltip: {
                                        valueSuffix: units[index]
                                    }
                                });
                            }
                        })

                        Charts[i_chart].chart = Highcharts.chart(Charts[i_chart].id, Charts[i_chart].highcharts);

                    }
                }

                setTimeout(() => {
                    for (let i_chart = 0; i_chart < Charts.length; i_chart++) {
                        if (Charts[i_chart] != undefined) {
                            Charts[i_chart].chart.reflow();
                        }
                    }
                }, 0);

                this.Charts = Charts;

                if(this.screen_width >= 768){
                    this.table.clear();
                }else{
                    this.table.forEach((table, index)  => {
                        this.table[index].clear();
                    })
                }

                if(this.data.dates){

                    if(this.screen_width >= 768){
                        this.data.dates.forEach((date, index) => {
                            let temp_row = ['<td>' + this.formatDate(date, 0) + '</td>'];

                            for (let sensor in this.data.measures) {

                                if (typeof this.data.measures[sensor][index] !== "undefined"){
                                    temp_row.push('<td>' + parseFloat(this.data.measures[sensor][index]) + '</td>');
                                } else {
                                    temp_row.push('<td></td>');
                                }

                            }

                            if (temp_row.length > 1)
                                this.table.row.add(temp_row).draw(false);

                        })

                    }else{

                        Object.values(this.data.measures).forEach((measure,  measuresIndex) => {
                            this.data.dates.forEach((date, index) => {

                                let temp_row = ['<td>' + this.formatDate(date, 0) + '</td>'];

                                if (typeof measure[index] !== "undefined"){
                                    temp_row.push('<td>' + parseFloat(measure[index]) + '</td>');
                                    this.table[measuresIndex].row.add(temp_row).draw(false);
                                }

                            })
                        })
                    }
                }

                this.Charts.forEach((chart, index) => {
                    if (chart != undefined) {

                        this.VMoptions[index] = new Array();

                        for (let id in chart.sensors) {

                            this.VMoptions[index].push({
                                id: chart.sensors[id],
                                name: this.data.names[chart.sensors[id]],
                            })
                        }

                        this.VMmodel[index] = this.VMoptions[index][0]

                        if (parseFloat(this.data.stats[chart.sensors[0]].max_amplitude) <= 0.1) {
                            this.charts_size.push(index);
                        }

                        this.max[index] = parseFloat(this.data.stats[chart.sensors[0]].max)
                        this.min[index] = parseFloat(this.data.stats[chart.sensors[0]].min)
                        this.max_amplitude[index] = parseFloat(this.data.stats[chart.sensors[0]].max_amplitude)
                        this.avg[index] = parseFloat(this.data.stats[chart.sensors[0]].avg)

                    }
                })

            }

        },

        openMapconfig() {
            this.Mapconfig = true;

            setTimeout(() => {

                // creating map
                this.mapModal = new mapboxgl.Map({
                attributionControl: false,
                container: "mapModal",
                style: 'mapbox://styles/datakorum-develop/cl4cu370w000v15tal53sxn9v',
                center: [this.gateway[0].GPS_lng, this.gateway[0].GPS_lat],
                zoom: 15,
                pitch: 50,
                bearing: 0,
                });

                this.markers = [];

                let img = document.createElement('img');
                img.src = this.str_img;

                let markerElement = document.createElement('div')
                markerElement.appendChild(img)

                this.markers.push(new mapboxgl.Marker({ element: markerElement })
                    .setLngLat([this.gateway[0].GPS_lng, this.gateway[0].GPS_lat])
                    .addTo(this.mapModal));

                this.mapModal.on("click", (event) => {

                    this.gateway[0].GPS_lat = parseFloat(event.lngLat.lat).toFixed(6)
                    this.gateway[0].GPS_lng = parseFloat(event.lngLat.lng).toFixed(6)

                    this.mapModal.flyTo({
                        center: [this.gateway[0].GPS_lng, this.gateway[0].GPS_lat],
                        essential: true,
                    });

                    // eliminar todos los marcadores existentes
                    this.markers.forEach(marker => marker.remove());
                    this.markers = [];

                    this.markers.push(new mapboxgl.Marker({ element: markerElement })
                        .setLngLat([this.gateway[0].GPS_lng, this.gateway[0].GPS_lat])
                        .addTo(this.mapModal));

                });

            }, 0);

        },

        updateLocation(){

            this.gateway[0].GPS_lat = document.getElementById('id_map_lat').value
            this.gateway[0].GPS_lng = document.getElementById('id_map_lng').value

            this.mapModal.flyTo({
                center: [this.gateway[0].GPS_lng, this.gateway[0].GPS_lat],
                essential: true,
            });

            // eliminar todos los marcadores existentes
            this.markers.forEach(marker => marker.remove());
            this.markers = [];

            let img = document.createElement('img');
            img.src = this.str_img;

            let markerElement = document.createElement('div')
            markerElement.appendChild(img)

            this.markers.push(new mapboxgl.Marker({ element: markerElement })
                .setLngLat([this.gateway[0].GPS_lng, this.gateway[0].GPS_lat])
                .addTo(this.mapModal));

        },

        SaveMap(){

            if(this.gateway[0].GPS_lat != this.gateway_update.GPS_lat || this.gateway[0].GPS_lng != this.gateway_update.GPS_lng){

                const data = {
                    name: this.data.name,
                    GPS_lat: this.gateway[0].GPS_lat,
                    GPS_lng: this.gateway[0].GPS_lng
                };

                window.emitter.emit('changeLoaderStatus', true, 'orbit')

                axios({
                    method: 'put',
                    url: window.location.origin + '/api/gateway/edit/' + this.user.id + '/' + this.gateway[0].id,
                    headers: {
                        'Accept': 'application/json',
                        'Content-Type': 'application/json',
                        'Authorization': 'Bearer ' + this.user.token
                    },
                    data: data
                })
                    .then((response) => {

                        this.gateway[0].GPS_lat = GPS_lat
                        this.gateway[0].GPS_lng = GPS_lng

                        this.gateway_update.GPS_lat = GPS_lat
                        this.gateway_update.GPS_lng = GPS_lng

                        window.emitter.emit('changeLoaderStatus', false, 'orbit')

                    })
                    .catch((error) => {
                        window.emitter.emit('changeLoaderStatus', false, 'orbit')

                        if(error.response && error.response.status && error.response.status == 429){
                        window.alert( this.$t('text.Api attempts'));
                        }else{
                        return "Failed!", error;
                        }

                    });
                this.rechargePage();

            }else {

                this.gateway[0].GPS_lat = this.gateway_update.GPS_lat
                this.gateway[0].GPS_lng = this.gateway_update.GPS_lng

            }

            this.Mapconfig = false;

        },

        CancelMap() {
            this.Mapconfig = false;

            if(this.gateway[0].GPS_lat != this.gateway_update.GPS_lat || this.gateway[0].GPS_lng != this.gateway_update.GPS_lng){
                this.gateway[0].GPS_lat = this.gateway_update.GPS_lat;
                this.gateway[0].GPS_lng = this.gateway_update.GPS_lng;
                this.rechargePage();
            }
        },

        check_VMoptions(index) {

            let times = 0;

            this.VMoptions[index].forEach((value, i) => {
                if (typeof value == 'object') {
                    times += 1;
                }
            })

            if (times > 1) {
                return true;
            } else {
                return false;
            }

        },

        selectMultiselectSensor: function (item) {

            this.Charts.forEach((chart, index_c) => {

                if (chart != undefined) {

                    chart.sensors.forEach((sensor, index_s) => {

                        if (sensor == item.id) {

                            this.max[index_c] = parseFloat(this.data.stats[sensor].max)
                            this.min[index_c] = parseFloat(this.data.stats[sensor].min)
                            this.max_amplitude[index_c] = parseFloat(this.data.stats[sensor].max_amplitude)
                            this.avg[index_c] = parseFloat(this.data.stats[sensor].avg)

                        }
                    })
                }
            })
        },

        Save_update(){

            const data = {
                name: document.getElementById('id_name').value,
                GPS_lat: document.getElementById('id_lat').value,
                GPS_lng: document.getElementById('id_lng').value,
            };

            window.emitter.emit('changeLoaderStatus', true, 'orbit')

            axios({
                method: 'put',
                url: window.location.origin + '/api/gateway/edit/' + this.user.id + '/' + this.gateway[0].id,
                headers: {
                    'Accept': 'application/json',
                    'Content-Type': 'application/json',
                    'Authorization': 'Bearer ' + this.user.token
                },
                data: data
            })
                .then((response) => {

                    this.gateway[0].name = data.name
                    this.gateway[0].GPS_lat = data.GPS_lat
                    this.gateway[0].GPS_lng = data.GPS_lng

                    this.gateway_update = {
                        name: data.name,
                        GPS_lat: data.GPS_lat,
                        GPS_lng: data.GPS_lng
                    }

                    this.rechargePage();

                    window.emitter.emit('changeLoaderStatus', false, 'orbit')

                })
                .catch((error) => {
                    window.emitter.emit('changeLoaderStatus', false, 'orbit')

                    if(error.response && error.response.status && error.response.status == 429){
                    window.alert( this.$t('text.Api attempts'));
                    }else{
                    return "Failed!", error;
                    }
                });
        },

        Cancel_update(){

            document.getElementById('id_name').value = this.gateway_update.name;
            document.getElementById('id_lat').value = parseFloat(this.gateway_update.GPS_lat).toFixed(6);
            document.getElementById('id_lng').value = parseFloat(this.gateway_update.GPS_lng).toFixed(6);

        },

        getName(unit) {

            let name;

            this.Charts.forEach((chart, index_c) => {
                if(chart != undefined){
                    if(chart.sensors.length <= 2){
                        for (let idfield in this.data.units) {
                            if(this.data.units[idfield] === unit){
                                name = this.data.names[idfield]
                            }
                        }
                    }else{
                        switch (unit) {
                            case 'ºC': name = 'Temperature'; break;
                            case '%': name = 'Humidity'; break;
                            default: name = '';
                        }
                    }
                }
            })

            return name;
        },

        methodFormatDatetoDatepicker() {
            return generateDateDatepicker(this.date)
        },

        format() {
            var date = this.date;
            var to = this.$t("general." + "To")

            var day1 = date[0].getDate();
            var month1 = date[0].getMonth() + 1;
            var year1 = date[0].getFullYear();
            var hour1 = date[0].getHours();
            var min1 = date[0].getMinutes();

            if (month1.toString().length == 1) month1 = '0' + month1;
            if (day1.toString().length == 1) day1 = '0' + day1;
            if (hour1.toString().length == 1) hour1 = '0' + hour1;
            if (min1.toString().length == 1) min1 = '0' + min1;

            var day2 = date[1].getDate();
            var month2 = date[1].getMonth() + 1;
            var year2 = date[1].getFullYear();
            var hour2 = date[1].getHours();
            var min2 = date[1].getMinutes();

            if (month2.toString().length == 1) month2 = '0' + month2;
            if (day2.toString().length == 1) day2 = '0' + day2;
            if (hour2.toString().length == 1) hour2 = '0' + hour2;
            if (min2.toString().length == 1) min2 = '0' + min2;

            return `${year1}/${month1}/${day1} ${hour1}:${min1} ${to} ${year2}/${month2}/${day2} ${hour2}:${min2}`;
        },

        generateDateDatepicker(date) {
            var day1 = date[0].getDate();
            var month1 = date[0].getMonth() + 1;
            var year1 = date[0].getFullYear();
            var hour1 = date[0].getHours();
            var min1 = date[0].getMinutes();

            if(month1.toString().length == 1) month1 = '0' + month1;
            if(day1.toString().length == 1) day1 = '0' + day1;
            if(hour1.toString().length == 1) hour1 = '0' + hour1;
            if(min1.toString().length == 1) min1 = '0' + min1;

            var day2 = date[1].getDate();
            var month2 = date[1].getMonth() + 1;
            var year2 = date[1].getFullYear();
            var hour2 = date[1].getHours();
            var min2 = date[1].getMinutes();

            if(month2.toString().length == 1) month2 = '0' + month2;
            if(day2.toString().length == 1) day2 = '0' + day2;
            if(hour2.toString().length == 1) hour2 = '0' + hour2;
            if(min2.toString().length == 1) min2 = '0' + min2;

            return `${year1}/${month1}/${day1} ${hour1}:${min1} `+ this.$t("general.To") +` ${year2}/${month2}/${day2} ${hour2}:${min2}`;
        },

        methodUpdateDate(modelData) {

            let date_start, date_end;

            if( modelData[0].getFullYear() === modelData[1].getFullYear() &&
                modelData[0].getMonth() === modelData[1].getMonth() &&
                modelData[0].getDate() === modelData[1].getDate()){

                modelData[0].setHours(0);
                modelData[0].setMinutes(0);
                modelData[0].setSeconds(0);

                modelData[1].setHours(23);
                modelData[1].setMinutes(59);
                modelData[1].setSeconds(59);

                date_start = this.formatDate(modelData[0], -1);
                date_end = this.formatDate(modelData[1], -1);

            }else{
                date_start = this.formatDate(modelData[0], -1);
                date_end = this.formatDate(modelData[1], -1);
            }

            window.emitter.emit('changeLoaderStatus', true, 'orbit')

            axios({
                method: 'get',
                url: window.location.origin + '/api/gateway/city/data_summary_readings/' + this.gateway[0].id + '/' + this.user.id + '/' + date_start + '/' + date_end + '/' + this.user.timezone_offset,
                headers: {
                    'Accept': 'application/json',
                    'Content-Type': 'application/json',
                    'Authorization': 'Bearer ' + this.user.token
                },
            })
                .then(response => {

                    window.emitter.emit('changeLoaderStatus', false, 'orbit')

                    this.data.measures = response.data.measures;
                    this.data.dates = response.data.dates;
                    this.data.stats = response.data.stats;
                    this.data.summary = response.data.summary;
                    this.rechargePage();
                })
                .catch((error) => {
                    window.emitter.emit('changeLoaderStatus', false, 'orbit')

                    if(error.response && error.response.status && error.response.status == 429){
                    window.alert( this.$t('text.Api attempts'));
                    }else{
                    return "Failed!", error;
                    }

                });
            },
    },
}
</script>

